@@ -1,6 +1,7 @@
 #' spatially or temporally aggregate stars object
 #' 
 #' spatially or temporally aggregate stars object, returning a data cube with lower spatial or temporal resolution 
+#' 
 #' @param x object of class \code{stars} with information to be aggregated
 #' @param by object of class \code{sf} or \code{sfc} for spatial aggregation, for temporal aggregation a vector with time values (\code{Date}, \code{POSIXct}, or \code{PCICt}) that is interpreted as a sequence of left-closed, right-open time intervals or a string like "months", "5 days" or the like (see \link{cut.POSIXt}); if by is an object of class \code{stars}, it is converted to sfc by \code{st_as_sfc(by, as_points = FALSE)} thus ignoring its time component.
 #' @param FUN aggregation function, such as \code{mean}
@@ -12,6 +13,7 @@
 #' @param as_points see \link[stars]{st_as_sf}: shall raster pixels be taken as points, or small square polygons?
 #' @param exact logical; if \code{TRUE}, use \link[exactextractr]{coverage_fraction} to compute exact overlap fractions of polygons with raster cells
 #' @export
+#' @aliases aggregate
 #' @examples
 #' # aggregate time dimension in format Date
 #' tif = system.file("tif/L7_ETMs.tif", package = "stars")
@@ -45,6 +47,21 @@
 #' nc = st_transform(nc, st_crs(prec_slice))
 #' agg = aggregate(prec_slice, st_geometry(nc), mean)
 #' plot(agg)
+#'
+#' # example of using a function for "by": aggregate by month-of-year
+#' d = c(10, 10, 150)
+#' a = array(rnorm(prod(d)), d) # pure noise
+#' times = Sys.Date() + seq(1, 2000, length.out = d[3])
+#' m = as.numeric(format(times, "%m"))
+#' signal = rep(sin(m / 12 * pi), each = prod(d[1:2])) # yearly period
+#' s = (st_as_stars(a) + signal) %>%
+#'       st_set_dimensions(3, values = times)
+#' f = function(x, format = "%B") {
+#' 	  months = format(as.Date(paste0("01-", 1:12, "-1970")), format)
+#' 	  factor(format(x, format), levels = months)
+#' }
+#' agg = aggregate(s, f, mean)
+#' plot(agg)
 aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects, 
 		as_points = any(st_dimension(by) == 2, na.rm = TRUE), rightmost.closed = FALSE,
 		left.open = FALSE, exact = FALSE) {
@@ -54,8 +71,8 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 		# and if not, then use st_normalize(by)
 	}
 
-	classes = c("sf", "sfc", "POSIXct", "Date", "PCICt", "character")
-	if (!inherits(by, classes))
+	classes = c("sf", "sfc", "POSIXct", "Date", "PCICt", "character", "function")
+	if (!is.function(by) && !inherits(by, classes))
 		stop(paste("currently, only `by' arguments of class", 
 			paste(classes, collapse= ", "), "supported"))
 
@@ -73,11 +90,13 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 		st = do.call(raster::stack, e)
 		m = raster::getValues(st)
 		if (identical(FUN, mean))
-			m = sweep(m, 2, colSums(m), "/")
+			m = sweep(m, 2, colSums(m), "/") # mean: divide weights by the sum of weights
 		else if (!identical(FUN, sum))
 			stop("for exact=TRUE, FUN should either be mean or sum")
 		new_dim = c(prod(dim(x)[1:2]), prod(dim(x)[-(1:2)]))
 		out_dim = c(ncol(m), dim(x)[-(1:2)])
+		if (isTRUE(list(...)$na.rm))
+			x = lapply(x, function(y) { y[is.na(y)] = 0.0; y })
 		agg = lapply(x, function(a) array(t(m) %*% array(a, dim = new_dim), dim = out_dim))
 		ret = st_as_stars(agg, dimensions = 
 			create_dimensions(append(list(sfc = create_dimension(values = by)),
@@ -119,13 +138,17 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 			ndims = 1
 			x = st_upfront(x, which_time(x))
 			values = expand_dimensions(x)[[1]]
-			if (inherits(by, "character")) {
+			if (inherits(by, "function")) {
+				i = by(values)
+				if (!is.factor(i))
+					i = as.factor(i)
+				by = levels(i)
+			} else if (inherits(by, "character")) {
 				i = cut(values, by, right = left.open)
 				by = if (inherits(values, "Date"))
 						as.Date(levels(i))
 					else
 						as.POSIXct(levels(i))
-				i = as.integer(i)
 			} else {
 				if (!inherits(values, class(by)))
 					warning(paste0('argument "by" is of a different class (', class(by)[1], 
@@ -133,7 +156,7 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 				i = findInterval(values, by, left.open = left.open, rightmost.closed = rightmost.closed)
 				i[ i == 0 | i == length(by) ] = NA
 			}
-			i
+			as.integer(i)
 		}
 
 	d = st_dimensions(x)
@@ -158,7 +181,7 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 
 	# reconstruct dimensions table:
 	d[[1]] = create_dimension(values = by)
-	names(d)[1] = if (inherits(by, c("POSIXct", "Date", "PCICt")))
+	names(d)[1] = if (inherits(by, c("POSIXct", "Date", "PCICt", "function")))
 			"time"
 		else
 			geom
@@ -194,6 +217,6 @@ aggregate.stars_proxy = function(x, by, FUN, ...) {
 	by = st_geometry(by)
 
 	# this assumes the result is small, no need to proxy
-	l = lapply(seq_along(by), function(i) aggregate(st_as_stars(x[by[i]]), by[i], FUN, ...))
+	l = lapply(seq_along(by), function(i) aggregate(st_normalize(st_as_stars(x[by[i]])), by[i], FUN, ...))
 	do.call(c, c(l, along = list(which_sfc(l[[1]]))))
 }
